﻿########
#LicAdmin v0.1
#Copyright:     Free to use, please leave this header intact
#Author:        Jos Lieben (OGD)
#Company:       OGD (http://www.ogd.nl)
#Script help:   http://www.liebensraum.nl
#Purpose:       
########
#Requirements:
########
#MS Online Services Signin Assistant:
#https://www.microsoft.com/en-us/download/details.aspx?id=41950
#Azure AD Module (x64):
#http://social.technet.microsoft.com/wiki/contents/articles/28552.microsoft-azure-active-directory-powershell-module-version-release-history.aspx

$o365login     = "xxx@lieben.nu"           #Username of O365 Admin
$o365pw        = "MyPassword123"                          #Password of O365 Admin
$logfile       = ($env:APPDATA + "\LicAdmin365.log")	#Logfile in case of errors
$e3SkuId       = "ENTERPRISEPACK_FACULTY"               #SkuId of the E3 license, you can get a list of these with Get-MsolAccountSku
$e1SkuId       = "STANDARDWOFFPACK_STUDENT"             #SkuId of the E1 license, you can get a list of these with Get-MsolAccountSku

#Start script
ac $logfile "-----$(Get-Date) LicAdmin365 v0.1 $($env:COMPUTERNAME) Session log-----`n"

#build Credential Object
$secpasswd = ConvertTo-SecureString $o365pw -AsPlainText -Force
$Credentials = New-Object System.Management.Automation.PSCredential ($o365login, $secpasswd)

#Load modules
$env:PSModulePath += ";C:\Windows\System32\WindowsPowerShell\v1.0\Modules\"
try{
    Import-Module msonline
}catch{
    ac $logfile "Critical error, unable to load Azure AD module, did you install it?"
    ac $logfile $error[0]
    Exit
}
try{
    Import-Module activedirectory
}catch{
    ac $logfile "Critical error, unable to load ActiveDirectory module, did you install it?"
    ac $logfile $error[0]
    Exit
}

#connect to MSOL
try{
    Connect-MsolService -Credential $Credentials
}catch{
    ac $logfile "Critical error, unable to connect to O365, check the credentials"
    ac $logfile $error[0]
    Exit
}

#store available licenses
$licenses = Get-MsolAccountSku

$e3licenses = $licenses | where-object {$_.AccountSkuId -like "*$($e3SkuId)*"}
$e1licenses = $licenses | where-object {$_.AccountSkuId -like "*$($e1SkuId)*"}

$e3licenses | Add-Member Available ($e3licenses.ActiveUnits - $e3licenses.ConsumedUnits)
$e1licenses | Add-Member Available ($e1licenses.ActiveUnits - $e1licenses.ConsumedUnits)

ac $logfile "Current licensing status:"
ac $logfile "E1 Licences used: $($e1licenses.ConsumedUnits) / $($e1licenses.ActiveUnits)"
ac $logfile "E3 Licences used: $($e3licenses.ConsumedUnits) / $($e3licenses.ActiveUnits)"

#get all active users with cao as attribute type, limit the returned amount by available licenses, ordered by date hired
try{
    $users_e3 = Get-AdUser -Filter 'employeeType -eq "cao" -and Enabled -eq $True' -Properties * -ResultSetSize ($e3licenses.ActiveUnits-1) | Sort-Object -property whenCreated
}catch{
    ac $logfile "Critical Error while attempting to query Active Directory"
    ac $logfile $error[0]
    Exit
}
if($users_e3.Count -lt 1){
    ac $logfile "Critical error: Active Directory query did not return any results!"
    ac $logfile $error[0]
    Exit
}

#loop over all users in Office 365
get-msoluser -All | % {
    #Prep variables
    $upn = $_.UserPrincipalName
    $e3Allowed = $False
    $e3Found = $False
    $e1Found = $False
    $e3Added = $False

    #check if user is in list of E3 allowed users
    if($users_3 | where-object{$_.UserPrincipalName -eq $upn}) {$e3Allowed = $True}

    #check licenses ready assigned
    foreach($lic in $_.Licenses){
        if($lic.AccountSkuID -like "*$($e3SkuId)*"){
            $e3Found = $True
        }
        if($lic.AccountSkuID -like "*$($e1SkuId)*"){
            $e1Found = $True
        }
    }
    #Remove E3 License if not allowed and currently assigned
	if($e3Found -eq $True -and $e3Allowed -ne $True){
        try{
            Set-MsolUserLicense -UserPrincipalName $upn -RemoveLicenses $e3licenses.AccountSkuId
            ac $logfile "E3 license removed for $upn"
        }catch{
            ac $logfile "Unable to remove E3 license from $upn"
            ac $logfile $error[0]
        }
        
    }
    #Add E3 License if not assigned but allowed
    if($e3Found -eq $False -and $e3Allowed -eq $True){
        try{
            Set-MsolUserLicense -UserPrincipalName $upn -AddLicenses $e3licenses.AccountSkuId
            ac $logfile "E3 license added to $upn" 
            if($e1Found){
                Set-MsolUserLicense -UserPrincipalName $upn -RemoveLicenses $e1licenses.AccountSkuId
                ac $logfile "E1 license removed from $upn" 
            }
        }catch{
            ac $logfile "Unable to add E3 license to or remove E1 license from $upn"
            ac $logfile $error[0]
            $e3Allowed -eq $False
        }      
    }
    #Add E1 License if not present and not E3
    if($e1Found -eq $False -and $e3Allowed -ne $True){
        try{
            Set-MsolUserLicense -UserPrincipalName $upn -AddLicenses $e1licenses.AccountSkuId
            ac $logfile "E1 license added to $upn"
        }catch{
            ac $logfile "Unable to add E1 license to $upn"
            ac $logfile $error[0]
        }        
    }
    
}

#end script
ac $logfile "-----$(Get-Date) LicAdmin365 v0.1 $($env:COMPUTERNAME) Session END-----`n" 


